@@ -1,5 +1,5 @@
 /* GKrellMSS - GKrellM Sound Scope
-|  Copyright (C) 2001 Bill Wilson
+|  Copyright (C) 2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |
@@ -28,24 +28,15 @@
 #include "../pixmaps/krell_sensitivity.xpm"
 
 /* ======================================================================== */
-#define CONFIG_NAME		"Sound Scope"
-#define CONFIG_KEYWORD	"sound"
-#define STYLE_NAME		"sound"
-
-#define	LOW_SWEEP	100
-#define	HIGH_SWEEP	50000
-
 #define DEFAULT_CHART_HEIGHT 30
 
 SoundMonitor		*sound;
-Oscope				*oscope;
 
-static Monitor		*mon_scope;
+static Monitor		*mon_sound;
 Chart				*chart;
 static ChartConfig	*chart_config;
 
 static gint			style_id;
-static gboolean		extra_info = TRUE;
 
 static GdkPixmap	*button_sweep_up_pixmap,
 					*button_sweep_dn_pixmap;
@@ -55,293 +46,79 @@ static GdkBitmap	*button_sweep_up_mask,
 static Decal		*decal_sweep_up,
 					*decal_sweep_dn;
 
-static Decal		*decal_usec;
+static Decal		*decal_mode_button;
+
+Decal				*decal_label0,	/* Chart labels */
+					*decal_label1;
 
 static Krell		*krell_in_motion;
 
 static gint			vu_meter_width;
 
 static void
-oscope_horizontal_scaling(void)
+draw_mode_button(gboolean pressed)
 	{
-	gchar	buf[32];
+	gint	frame, x;
 
-	oscope->t_sample = 1 / (gfloat) SAMPLE_RATE;
-	oscope->dx_per_point = 0;
-	do
-		{
-		++oscope->dx_per_point;
-		oscope->t_trace = (gfloat) oscope->usec_per_div * 1e-6;
-		oscope->t_trace *= HORIZONTAL_DIVS;
-		oscope->samples_per_point = oscope->t_trace / oscope->t_sample
-					/ (gfloat) chart->w * (gfloat) oscope->dx_per_point;
-		}
-	while (oscope->samples_per_point < 1);
-
-	if (oscope->usec_per_div >= 1000)
-		sprintf(buf, "%d msec", oscope->usec_per_div / 1000);
-	else
-		sprintf(buf, "%d usec", oscope->usec_per_div);
-	gkrellm_draw_decal_text(NULL, decal_usec, buf, -1);
-	}
-
-static void
-trigger_delay(gint channel)
-	{
-	SoundSample	*ss = sound->buffer;
-	gfloat		flimit;
-	gint		y0, i, n, limit, trigger, delay;
-	
-	trigger = (gint) (oscope->vert_trigger * oscope->vert_max);
-	delay = -2;
-	flimit = oscope->samples_per_point;
-	limit = flimit;
-	for (i = 0; i < sound->buf_count - limit; )
-		{
-		for (y0 = 0, n = 0; n < limit; ++n)
-			{
-			if (channel == CHANNEL_L)
-				y0 += ss[i].left;
-			else if (channel == CHANNEL_R)
-				y0 += ss[i].right;
-			else if (channel == CHANNEL_LR)
-				y0 += (ss[i].left + ss[i].right) / 2;
-			}
-		y0 = y0 / limit;
-		if (y0 < trigger)
-			delay = -1;
-		if (y0 >= trigger && delay == -1)
-			{
-			delay = i;
-			break;
-			}
-		flimit += oscope->samples_per_point;
-		i = flimit;
-		}
-	sound->buf_index = (delay < 0) ? 0 : delay;
-	}
-
-
-void
-draw_oscope_line_trace(gint channel)
-	{
-	SoundSample	*ss = sound->buffer;
-	gfloat	flimit;
-	gint	x, x0, y0, y1, i, limit, n;
-
-	oscope->gc = gkrellm_draw_GC(1);
-	gdk_gc_set_foreground(oscope->gc, gkrellm_in_color());
-
-	y1 = oscope->y_append;
-	x0 = oscope->x_append;
-	oscope->x_append = 0;
-	oscope->y_append = 0;
-	flimit = sound->buf_index + oscope->samples_per_point;
-	for (i = sound->buf_index, x = x0; x < chart->w; x += oscope->dx_per_point)
-		{
-		limit = flimit;
-		if (limit >= sound->buf_count - 1)
-			{
-			oscope->y_append = y1;
-			oscope->x_append = x;
-			break;
-			}
-		for (n = 0, y0 = 0; i < limit; ++i, ++n)
-			{
-			if (channel == CHANNEL_L)
-				y0 += ss[i].left;
-			else if (channel == CHANNEL_R)
-				y0 += ss[i].right;
-			else if (channel == CHANNEL_LR)
-				y0 += (ss[i].left + ss[i].right) / 2;
-			}
-		y0 /= n;
-		y0 = (chart->h / 2) * -y0 / oscope->vert_max;
-		y0 += chart->h / 2;
-		if (x > 0)
-			{
-			gdk_draw_line(chart->pixmap, oscope->gc,
-					x - oscope->dx_per_point, y1, x, y0);
-			}
-		y1 = y0;
-		flimit += oscope->samples_per_point;
-		}
-	sound->buf_index = 0;
-	sound->buf_count = 0;
+	x = gkrellm_chart_width() - decal_mode_button->w + 1;
+	frame = pressed ? D_MISC_BUTTON_IN : D_MISC_BUTTON_OUT;
+	gkrellm_draw_decal_pixmap(NULL, decal_mode_button, frame);
+	gkrellm_draw_decal_on_chart(chart, decal_mode_button, x, 0);
 	}
 
 void
-draw_oscope_bar_trace(gint channel)
+draw_sound_chart(gboolean force_reset, gboolean draw_grid)
 	{
-	SoundSample	*ss = sound->buffer;
-	gfloat	flimit;
-	gint	x, x0, y, y0, y1, i, limit, n;
-	gint	y0_prev, y1_prev;
-
-	oscope->gc = gkrellm_draw_GC(1);
-	gdk_gc_set_foreground(oscope->gc, gkrellm_in_color());
-
-	y = y0 = y1 = 0;
-	y0_prev = y1_prev = oscope->y_append;
-	x0 = oscope->x_append;
-	oscope->x_append = 0;
-	oscope->y_append = 0;
-	flimit = sound->buf_index + oscope->samples_per_point;
-	for (i = sound->buf_index, x = x0; x < chart->w; x += oscope->dx_per_point)
-		{
-		limit = flimit;
-		if (limit >= sound->buf_count - 1)
-			{
-			oscope->y_append = (y1_prev + y0_prev) / 2;
-			oscope->x_append = x;
-			break;
-			}
-		for (n = 0; i < limit; ++i, ++n)
-			{
-			if (channel == CHANNEL_L)
-				y = ss[i].left;
-			else if (channel == CHANNEL_R)
-				y = ss[i].right;
-			else if (channel == CHANNEL_LR)
-				y = (ss[i].left + ss[i].right) / 2;
-			else
-				break;
-			if (n == 0)
-				y0 = y1 = y;
-			else
-				{
-				if (y < y0)
-					y0 = y;
-				if (y > y1)
-					y1 = y;
-				}
-			if (x > 0)
-				{
-				if (y0 > y1_prev)
-					y0 = y1_prev;
-				if (y1 < y0_prev)
-					y1 = y0_prev;
-				}
-			}
-		y0_prev = y0;
-		y1_prev = y1;
-
-		y0 = (chart->h / 2) * -y0 / oscope->vert_max;
-		y1 = (chart->h / 2) * -y1 / oscope->vert_max;
-		y0 += chart->h / 2;
-		y1 += chart->h / 2;
-		gdk_draw_line(chart->pixmap, oscope->gc, x, y1, x, y0);
-		flimit += oscope->samples_per_point;
-		}
-	sound->buf_index = 0;
-	sound->buf_count = 0;
-	}
-
-void
-oscope_trace(gint channel)
-	{
-	if (oscope->dx_per_point > 1)
-		draw_oscope_line_trace(channel);
-	else
-		draw_oscope_bar_trace(channel);
-	}
-
-static void
-draw_oscope_grid(void)
-	{
-	GdkImage	*grid_image;
-	GdkGC		*gc;
-	GdkColor	color;
-	gint		h, x, dx;
-
-	/* Draw grid lines on bg_src_pixmap so gkrellm_clear_chart_pixmap()
-	|  will include grid lines.
-	*/
-	gkrellm_clean_bg_src_pixmap(chart);
-	gkrellm_draw_chart_grid_line(chart, chart->bg_src_pixmap, chart->h / 4);
-	gkrellm_draw_chart_grid_line(chart, chart->bg_src_pixmap, chart->h / 2);
-	gkrellm_draw_chart_grid_line(chart, chart->bg_src_pixmap, 3 * chart->h/4);
-
-	/* There is no vertical grid image, so I make a vertical grid line using
-	|  pixel values from the horizontal grid image at desired x positions.
-	*/
-	h = ((GdkWindowPrivate *) chart->bg_grid_pixmap)->height;
-	grid_image = gdk_image_get(chart->bg_grid_pixmap, 0, 0,
-					gkrellm_chart_width(), h);
-	gc = gkrellm_draw_GC(3);
-	dx = chart->w / HORIZONTAL_DIVS;
-	for (x = dx; x < HORIZONTAL_DIVS * dx; x += dx)
-		{
-		color.pixel = gdk_image_get_pixel(grid_image, x, 0);
-		gdk_gc_set_foreground(gc, &color);
-		gdk_draw_line(chart->bg_src_pixmap, gc, x - 1, 0, x - 1, chart->h - 1);
-		if (h > 1)
-			{
-			color.pixel = gdk_image_get_pixel(grid_image, x, 1);
-			gdk_gc_set_foreground(gc, &color);
-			gdk_draw_line(chart->bg_src_pixmap, gc, x, 0, x, chart->h - 1);
-			}
-		}
-	gdk_image_destroy(grid_image);
-	}
-
-static void
-draw_chart_to_screen(void)
-	{
-	if (sound->fd < 0)
-		gkrellm_draw_chart_text(chart, DEFAULT_STYLE_ID,
-			"\\cNo esd?\\n\\f\\cClick here to\\n\\f\\ctry to open");
-	else if (extra_info)
-		gkrellm_draw_decal_on_chart(chart, decal_usec, 2,
-					chart->h - decal_usec->h);
+	if (sound->mode == SOUND_MODE_OSCOPE)
+		draw_oscope(force_reset, draw_grid);
+	else if (sound->mode == SOUND_MODE_SPECTRUM)
+		draw_spectrum(force_reset, draw_grid);
+	if (sound->mouse_in_chart)
+		draw_mode_button(sound->mode_button_pressed);
 	gkrellm_draw_chart_to_screen(chart);
-	}
-
-static void
-reset_oscope(gboolean force)
-	{
-	gint	y;
-
-	if (oscope->dirty || force || (GK.two_second_tick && sound->fd < 0))
-		{
-		y = chart->h / 2;
-		gkrellm_clear_chart_pixmap(chart);
-		oscope->gc = gkrellm_draw_GC(1);
-		gdk_gc_set_foreground(oscope->gc, gkrellm_in_color());
-		gdk_draw_line(chart->pixmap, oscope->gc, 0, y, chart->w - 1, y);
-		draw_chart_to_screen();
-		}
-	oscope->dirty = FALSE;
-	sound->buf_count = 0;
-	sound->buf_index = 0;
-	oscope->x_append = oscope->y_append = 0;
-	}
-
-static void
-draw_oscope(void)
-	{
-	gkrellm_clear_chart_pixmap(chart);
-	trigger_delay(CHANNEL_LR);
-	oscope_trace(CHANNEL_LR);
-	draw_chart_to_screen();
-	oscope->dirty = TRUE;
+	sound->streaming = FALSE;
 	}
 
 static gint
-cb_extra(GtkWidget *widget, GdkEventButton *ev, gpointer data)
+cb_chart_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	{
-	if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
+	sound->mode_button_pressed = FALSE;
+	return TRUE;
+	}
+
+static gint
+cb_chart_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
+	{
+	if (gkrellm_in_decal(decal_mode_button, ev))
+		{
+		sound->mode += 1;
+		if (sound->mode >= N_SOUND_MODES)
+			sound->mode = 0;
+		sound->mode_button_pressed = TRUE;
+		gkrellm_config_modified();
+		draw_sound_chart(FORCE_RESET, DRAW_GRID);
+		}
+	else if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
+		{
+		if (sound->mode == SOUND_MODE_SPECTRUM && !spectrum->reset)
+			{
+			if (spectrum->x_highlight > 0)
+				spectrum->x_highlight = 0;
+			else
+				spectrum->x_highlight = sound->x_mouse;
+			}
+		}
+	else if (ev->button == 2)
 		{
 		if (sound->fd < 0)
 			{
 			sound_open_stream(sound->host);
-			oscope->dirty = TRUE;
 			}
 		else
 			{
-			extra_info = !extra_info;
+			sound->extra_info = !sound->extra_info;
 			gkrellm_config_modified();
-			reset_oscope(TRUE);
+			draw_sound_chart(FORCE_RESET, 0);
 			}
 		}
 	else if (   ev->button == 3
@@ -352,7 +129,7 @@ cb_extra(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	}
 
 static void
-update_scope(void)
+update_sound(void)
 	{
 	gint		d, left, right, left_peak, right_peak;
 
@@ -382,25 +159,23 @@ update_scope(void)
 	sound->left_value = 0;
 	sound->right_value = 0;
 
-	if (sound->krell_sensitivity_y_target > sound->krell_sensitivity_y)
+	d = sound->krell_sensitivity_y_target - sound->krell_sensitivity_y;
+	if (d > 0)
 		{
-		sound->krell_sensitivity_y += 1;
+		sound->krell_sensitivity_y += 1 + d / 4;
 		gkrellm_move_krell_yoff(chart->panel, sound->krell_sensitivity,
 					sound->krell_sensitivity_y);
 		}
-	else if (sound->krell_sensitivity_y_target < sound->krell_sensitivity_y)
+	else if (d < 0)
 		{
-		sound->krell_sensitivity_y -= 1;
+		sound->krell_sensitivity_y -= 1 - d / 4;
 		gkrellm_move_krell_yoff(chart->panel, sound->krell_sensitivity,
 					sound->krell_sensitivity_y);
 		}
 
 	gkrellm_draw_panel_layers(chart->panel);
 
-	if (!sound->streaming)
-		reset_oscope(FALSE);
-	else if (!oscope->x_append && sound->buf_count)
-		draw_oscope();
+	draw_sound_chart(0, 0);
 	sound->streaming = FALSE;
 	}
 
@@ -408,8 +183,12 @@ update_scope(void)
 static void
 sound_vertical_scaling(void)
 	{
-	oscope->vert_max = (gint) (oscope->vert_sensitivity * 32767.0);
+	gfloat	audio_taper;
 
+	audio_taper = log(1 + sound->vert_sensitivity) / log(2.0);
+
+	oscope->vert_max = (gint) (audio_taper * 32767.0);
+	spectrum->vert_max = (gint) (audio_taper * 24);
 	gkrellm_set_krell_full_scale(sound->krell_left, oscope->vert_max, 1);
 	gkrellm_set_krell_full_scale(sound->krell_right, oscope->vert_max, 1);
 	gkrellm_set_krell_full_scale(sound->krell_left_peak, oscope->vert_max, 1);
@@ -432,11 +211,11 @@ update_slider_position(Krell *k, gint x_ev)
 	if (x < 0)
 		x = 0;
 
-	oscope->vert_sensitivity = (100.0 - (gfloat) x) / 100.0;
-	if (oscope->vert_sensitivity < 0.05)
-		oscope->vert_sensitivity = 0.05;
-	if (oscope->vert_sensitivity > 1.0)
-		oscope->vert_sensitivity = 1.0;
+	sound->vert_sensitivity = (100.0 - (gfloat) x) / 100.0;
+	if (sound->vert_sensitivity < 0.05)
+		sound->vert_sensitivity = 0.05;
+	if (sound->vert_sensitivity > 1.0)
+		sound->vert_sensitivity = 1.0;
 	sound_vertical_scaling();
 	gkrellm_config_modified();
 	gkrellm_update_krell(chart->panel, k, (gulong) x);
@@ -461,6 +240,31 @@ expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	}
 
 
+static gboolean
+cb_chart_enter(GtkWidget *w, GdkEventButton *ev, gpointer data)
+	{
+	sound->mouse_in_chart = TRUE;
+	draw_mode_button(FALSE);
+	gkrellm_draw_chart_to_screen(chart);
+	return TRUE;
+	}
+
+static gint
+cb_chart_leave(GtkWidget *w, GdkEventButton *ev, gpointer data)
+	{
+	sound->mouse_in_chart = FALSE;
+	sound->mode_button_pressed = FALSE;
+	draw_sound_chart(FORCE_RESET, 0);
+	return TRUE;
+	}
+
+static gint
+cb_chart_motion(GtkWidget *widget, GdkEventButton *ev)
+	{
+	sound->x_mouse = ev->x;
+	return TRUE;
+	}
+
 static gint
 cb_panel_enter(GtkWidget *widget, GdkEventButton *ev)
 	{
@@ -503,7 +307,7 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 
 	if (ev->button == 3)
 		{
-		gkrellm_open_config_window(mon_scope);
+		gkrellm_open_config_window(mon_sound);
 		return TRUE;
 		}
 	if (   ev->button == 1 && ev->x > sound->vu_x0
@@ -541,8 +345,11 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 static void
 height_changed(gpointer data)
 	{
-	draw_oscope_grid();
-	reset_oscope(TRUE);
+	if (sound->mode == SOUND_MODE_SPECTRUM)
+		{
+		load_spectrum_images();
+		}
+	draw_sound_chart(FORCE_RESET, DRAW_GRID);
 	}
 
 static void
@@ -550,15 +357,25 @@ create_chart(GtkWidget *vbox, gint first_create)
 	{
 	Chart		*cp = chart;
 
-	/* Since decal_usec will be drawn on the chart, remove it from the panel.
+	/* Create some decals that will be drawn on the chart.  They won't live
+	|  in a panel where they can be automatically destroyed, so I must
+    |  destroy them at create events and then recreate them with NULL panel
+	|  and style pointers.
 	*/
-	gkrellm_destroy_decal(decal_usec);
-	decal_usec = gkrellm_create_decal_text(chart->panel, "888 msec",
+	gkrellm_destroy_decal(decal_label0);
+	gkrellm_destroy_decal(decal_label1);
+	decal_label0 = gkrellm_create_decal_text(NULL, "888 msec",
 			gkrellm_chart_alt_textstyle(DEFAULT_STYLE_ID), NULL, 2, 0, 0);
-	gkrellm_remove_decal(chart->panel, decal_usec);
+	decal_label1 = gkrellm_create_decal_text(NULL, "8.8K",
+			gkrellm_chart_alt_textstyle(DEFAULT_STYLE_ID), NULL, 2, 0, 0);
+
+	gkrellm_destroy_decal(decal_mode_button);
+	decal_mode_button = gkrellm_create_decal_pixmap(NULL,
+			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
+			N_MISC_DECALS, NULL, 0, 0);
 
 	gkrellm_set_chart_height_default(cp, DEFAULT_CHART_HEIGHT);
-	gkrellm_chart_create(vbox, mon_scope, cp, &chart_config);
+	gkrellm_chart_create(vbox, mon_sound, cp, &chart_config);
 
 	gkrellm_set_chartconfig_flags(chart_config, NO_CONFIG_FIXED_GRIDS);
 
@@ -573,12 +390,20 @@ create_chart(GtkWidget *vbox, gint first_create)
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "expose_event",
 				(GtkSignalFunc) expose_event, NULL);
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
-				"button_press_event", (GtkSignalFunc) cb_extra, cp);
+				"button_press_event", (GtkSignalFunc) cb_chart_press, cp);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
+				"button_release_event", (GtkSignalFunc) cb_chart_release, cp);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "enter_notify_event",
+				(GtkSignalFunc) cb_chart_enter, NULL);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "leave_notify_event",
+				(GtkSignalFunc) cb_chart_leave, NULL);
+		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
+				"motion_notify_event", (GtkSignalFunc) cb_chart_motion, NULL);
 		gtk_widget_show(vbox);
 		}
-	draw_oscope_grid();
 	oscope_horizontal_scaling();
-	reset_oscope(TRUE);
+	load_spectrum_images();
+	draw_sound_chart(FORCE_RESET, DRAW_GRID);
 	}
 
 static void
@@ -588,10 +413,15 @@ cb_sweep_pressed(DecalButton *button)
 
 	dir = GPOINTER_TO_INT(button->data);
 	dir = (dir == 0) ? 1 : -1;
-	oscope->usec_per_div = gkrellm_125_sequence(oscope->usec_per_div + dir,
+	if (sound->mode == SOUND_MODE_OSCOPE)
+		{
+		oscope->usec_per_div = gkrellm_125_sequence(oscope->usec_per_div + dir,
 				TRUE, LOW_SWEEP, HIGH_SWEEP, FALSE, FALSE);
-	oscope_horizontal_scaling();
-	reset_oscope(TRUE);
+		oscope_horizontal_scaling();
+		}
+	else if (sound->mode == SOUND_MODE_SPECTRUM)
+		change_spectrum_scale(dir);
+	draw_sound_chart(FORCE_RESET, 0);
 	}
 
 static void
@@ -599,25 +429,29 @@ create_panel_buttons(Panel *p)
 	{
 	GdkImlibImage	*im	= NULL;
 	Style			*style;
+	Margin			*margin;
 	gint			x, y;
 
 	style = gkrellm_meter_style(style_id);
+	margin = gkrellm_get_style_margins(style);
 
 	y = 3;
 	gkrellm_get_gkrellmrc_integer("sound_button_sweep_yoff", &y);
 
-	gkrellm_load_image("buttom_sweep_dn", button_sweep_dn_xpm,
-				&im, STYLE_NAME);
+	if (!gkrellm_load_image("buttom_sweep_dn", NULL, &im, STYLE_NAME))
+		gkrellm_load_image("button_sweep_dn", button_sweep_dn_xpm,
+					&im, STYLE_NAME);
 	gkrellm_render_to_pixmap(im, &button_sweep_dn_pixmap,
 				&button_sweep_dn_mask, 0, 0);
 	decal_sweep_dn = gkrellm_create_decal_pixmap(p, button_sweep_dn_pixmap,
-				button_sweep_dn_mask, 2, style, -1, y);
+				button_sweep_dn_mask, 2, style, margin->left, y);
 
 	x = decal_sweep_dn->x;
 	y = decal_sweep_dn->y;
 
-	gkrellm_load_image("buttom_sweep_up", button_sweep_up_xpm,
-				&im, STYLE_NAME);
+	if (!gkrellm_load_image("buttom_sweep_up", NULL, &im, STYLE_NAME))
+		gkrellm_load_image("button_sweep_up", button_sweep_up_xpm,
+					&im, STYLE_NAME);
 	gkrellm_render_to_pixmap(im, &button_sweep_up_pixmap,
 				&button_sweep_up_mask, 0, 0);
 	decal_sweep_up = gkrellm_create_decal_pixmap(p, button_sweep_up_pixmap,
@@ -632,97 +466,78 @@ create_panel_buttons(Panel *p)
 	vu_meter_width = gkrellm_chart_width() - sound->vu_x0;
 	}
 
+static Krell *
+default_or_themed_krell(Panel *p, gchar *name, GdkImlibImage *im, gint yoff,
+	gint depth, gint x_hot, gint expand, gint left_margin, gint right_margin)
+	{
+	Krell	*k;
+	Style	*style;
+
+	/* Work with a copy since we may modify the style.
+	*/
+	style = gkrellm_copy_style(gkrellm_meter_style_by_name(name));
+	if (!gkrellm_style_is_themed(style))
+		gkrellm_set_style_krell_values(style, yoff, depth, x_hot,
+				expand, left_margin, right_margin);
+	else
+		{
+		if (style->krell_left_margin == 0)
+			style->krell_left_margin = left_margin;
+		}
+	k = gkrellm_create_krell(p, im, style);
+	gkrellm_monotonic_krell_values(k, FALSE);
+
+	/* Unlike the style pointer passed to gkrellm_panel_configure(), the krells
+	|  don't need the style to persist.
+	*/
+	g_free(style);
+	return k;
+	}
+
 static void
 create_panel(GtkWidget *vbox, gint first_create)
 	{
 	GdkImlibImage	*im = NULL;
 	Panel			*p;
-	Style			*s;
-	gint			x, yoff;
+	gint			x;
 
 	p = chart->panel;
 	create_panel_buttons(p);
 	x = sound->vu_x0;
 
-	s = gkrellm_copy_style(gkrellm_meter_style(style_id));
-
-	s->krell_yoff = 3;
-	s->krell_depth = 1;
-	s->krell_x_hot = 59;
-	s->krell_expand = KRELL_EXPAND_LEFT;
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_depth", &s->krell_depth);
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_x_hot", &s->krell_x_hot);
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_expand", &s->krell_expand);
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_left_yoff", &s->krell_yoff);
 	gkrellm_load_image("krell_vu", krell_vu_xpm, &im, STYLE_NAME);
-	sound->krell_left = gkrellm_create_krell(p, im, s);
-	gkrellm_monotonic_krell_values(sound->krell_left, FALSE);
-	gkrellm_set_krell_margins(p, sound->krell_left, x, 0);
+	sound->krell_left = default_or_themed_krell(p, "sound.vu_left",
+				im, 3, 1, 59, KRELL_EXPAND_LEFT, x, 0);
+	sound->krell_right = default_or_themed_krell(p, "sound.vu_right",
+				im, 9, 1, 59, KRELL_EXPAND_LEFT, x, 0);
 
-	s->krell_yoff = 9;
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_right_yoff", &s->krell_yoff);
-	sound->krell_right = gkrellm_create_krell(p, im, s);
-	gkrellm_monotonic_krell_values(sound->krell_right, FALSE);
-	gkrellm_set_krell_margins(p, sound->krell_right, x, 0);
-
-	s->krell_yoff = 2;
-	s->krell_depth = 5;
-	s->krell_x_hot = -1;
-	s->krell_expand = KRELL_EXPAND_NONE;
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_peak_depth",
-				&s->krell_depth);
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_peak_x_hot",
-				&s->krell_x_hot);
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_peak_left_yoff",
-				&s->krell_yoff);
 	gkrellm_load_image("krell_vu_peak", krell_vu_peak_xpm, &im, STYLE_NAME);
-	sound->krell_left_peak = gkrellm_create_krell(p, im, s);
-	gkrellm_monotonic_krell_values(sound->krell_left_peak, FALSE);
-	gkrellm_set_krell_margins(p, sound->krell_left_peak, x, 0);
-
-	s->krell_yoff = 8;
-	gkrellm_get_gkrellmrc_integer("sound_krell_vu_peak_right_yoff",
-				&s->krell_yoff);
-	sound->krell_right_peak = gkrellm_create_krell(p, im, s);
-	gkrellm_monotonic_krell_values(sound->krell_right_peak, FALSE);
-	gkrellm_set_krell_margins(p, sound->krell_right_peak, x, 0);
+	sound->krell_left_peak = default_or_themed_krell(p, "sound.vu_left_peak",
+				im, 2, 5, -1, KRELL_EXPAND_NONE, x, 0);
+	sound->krell_right_peak = default_or_themed_krell(p, "sound.vu_right_peak",
+				im, 8, 5, -1, KRELL_EXPAND_NONE, x, 0);
 
 	sound_vertical_scaling();
 
-	sound->krell_sensitivity_y_up = -10;
-	sound->krell_sensitivity_y_dn = 0;
-	s->krell_yoff = 4;
-	s->krell_depth = 1;
-	s->krell_x_hot = -1;
-	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_depth",
-				&s->krell_depth);
-	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_x_hot",
-				&s->krell_x_hot);
-	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_y_up",
-				&sound->krell_sensitivity_y_up);
-	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_y_dn",
-				&sound->krell_sensitivity_y_dn);
 	gkrellm_load_image("krell_sensitivity", krell_sensitivity_xpm,
-            	&im, STYLE_NAME);
-	/* A negative y_up won't work in the create, so save its value and
-	|  do the real negative positioning later.
-	*/
-	yoff = s->krell_yoff = sound->krell_sensitivity_y_up;
-	sound->krell_sensitivity_y_target = s->krell_yoff;
-	sound->krell_sensitivity = gkrellm_create_krell(p, im, s);
-	gkrellm_monotonic_krell_values(sound->krell_sensitivity, FALSE);
-	gkrellm_set_krell_margins(p, sound->krell_sensitivity, x, 0);
+				&im, STYLE_NAME);
+	sound->krell_sensitivity = default_or_themed_krell(p, "sound.sensitivity",
+				im, 0, 1, -1, KRELL_EXPAND_NONE, x, 0);
 	gkrellm_set_krell_full_scale(sound->krell_sensitivity, 100, 1);
 
-	/* Unlike the style pointer passed to gkrellm_panel_configure(), the krells
-	|  don't need the style to persist.
-	*/
-	g_free(s);
+	sound->krell_sensitivity_y_up = -10;
+	sound->krell_sensitivity_y_dn = 0;
+	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_y_up",
+			&sound->krell_sensitivity_y_up);
+	gkrellm_get_gkrellmrc_integer("sound_krell_sensitivity_y_dn",
+			&sound->krell_sensitivity_y_dn);
+	sound->krell_sensitivity_y_target = sound->krell_sensitivity_y_up;
+
 	if (im)
 		gdk_imlib_kill_image(im);
 
 	gkrellm_panel_configure(p, NULL, gkrellm_meter_style(style_id));
-	gkrellm_panel_create(vbox, mon_scope, p);
+	gkrellm_panel_create(vbox, mon_sound, p);
 
 	gkrellm_make_decal_button(p, decal_sweep_dn, cb_sweep_pressed,
 				GINT_TO_POINTER(0), 1, 0);
@@ -745,19 +560,19 @@ create_panel(GtkWidget *vbox, gint first_create)
 				"leave_notify_event", (GtkSignalFunc) cb_panel_leave, NULL);
 		}
 	gkrellm_update_krell(chart->panel, sound->krell_sensitivity,
-				(gulong) (100.0 * (1.0 - oscope->vert_sensitivity)));
-	sound->x_sensitivity_raw = (gint) ((1.0 - oscope->vert_sensitivity)
+				(gulong) (100.0 * (1.0 - sound->vert_sensitivity)));
+	sound->x_sensitivity_raw = (gint) ((1.0 - sound->vert_sensitivity)
 				* (gfloat) vu_meter_width) + sound->vu_x0;
 
 	/* To get the negative y_up, must move the krell since the create
-	|  interprets negative offsets as special placement codes.  Also,
-	|  this does the initial draw for me.
+	|  interprets negative offsets as special placement codes.
 	*/
-	gkrellm_move_krell_yoff(p, sound->krell_sensitivity, yoff);
+	gkrellm_move_krell_yoff(p, sound->krell_sensitivity,
+				sound->krell_sensitivity_y_up);
 	}
 
 static void
-create_scope(GtkWidget *vbox, gint first_create)
+create_sound(GtkWidget *vbox, gint first_create)
 	{
 	if (first_create)
 		{
@@ -765,6 +580,7 @@ create_scope(GtkWidget *vbox, gint first_create)
 		chart->panel = gkrellm_panel_new0();
 		sound_open_stream(sound->host);
 		}
+	sound->gc = gkrellm_draw_GC(1);
 	create_chart(vbox, first_create);
 	create_panel(vbox, first_create);
 	}
@@ -773,9 +589,11 @@ create_scope(GtkWidget *vbox, gint first_create)
 static void
 save_config(FILE *f)
 	{
-	fprintf(f, "%s sensitivity %f\n", CONFIG_KEYWORD,oscope->vert_sensitivity);
+	fprintf(f, "%s mode %d\n", CONFIG_KEYWORD, sound->mode);
+	fprintf(f, "%s sensitivity %f\n", CONFIG_KEYWORD, sound->vert_sensitivity);
+	fprintf(f, "%s extra_info %d\n", CONFIG_KEYWORD, sound->extra_info);
 	fprintf(f, "%s usec_per_div %d\n", CONFIG_KEYWORD, oscope->usec_per_div);
-	fprintf(f, "%s extra_info %d\n", CONFIG_KEYWORD, extra_info);
+	fprintf(f, "%s spectrum_scale %d\n", CONFIG_KEYWORD,spectrum->scale_index);
 	gkrellm_save_chartconfig(f, chart_config, CONFIG_KEYWORD, NULL);
 	}
 
@@ -788,18 +606,22 @@ load_config(gchar *arg)
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n != 2)
 		return;
-	if (!strcmp(config, "sensitivity"))
+	if (!strcmp(config, "mode"))
+		sscanf(item, "%d", &sound->mode);
+	else if (!strcmp(config, "sensitivity"))
 		{
-		sscanf(item, "%f", &oscope->vert_sensitivity);
-		if (oscope->vert_sensitivity < 0.05)
-			oscope->vert_sensitivity = 0.05;
-		if (oscope->vert_sensitivity > 1.0)
-			oscope->vert_sensitivity = 1.0;
+		sscanf(item, "%f", &sound->vert_sensitivity);
+		if (sound->vert_sensitivity < 0.05)
+			sound->vert_sensitivity = 0.05;
+		if (sound->vert_sensitivity > 1.0)
+			sound->vert_sensitivity = 1.0;
 		}
+	else if (!strcmp(config, "extra_info"))
+		sscanf(item, "%d", &sound->extra_info);
 	else if (!strcmp(config, "usec_per_div"))
 		sscanf(item, "%d", &oscope->usec_per_div);
-	else if (!strcmp(config, "extra_info"))
-		sscanf(item, "%d", &extra_info);
+	else if (!strcmp(config, "spectrum_scale"))
+		sscanf(item, "%d", &spectrum->scale_index);
 	else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 		gkrellm_load_chartconfig(&chart_config, item, 0);
 	}
@@ -821,26 +643,36 @@ N_("Two fast response krells show left and right audio signal levels\n"
 	"response krells.\n"),
 "\n",
 
-N_("<b>Oscope Chart\n"),
-N_("Shows combined left and right audio signals as an oscilloscope trace.\n"),
+N_("<b>Chart\n"),
+N_("The chart has oscilloscope and spectrum analyzer modes toggled by\n"
+	"pressing the mode button on the chart.\n"),
 "\n",
 
-N_("<b>Oscope Sweep Speed Buttons\n"),
-N_("The two buttons to the left of the VU Meter select an oscope horizontal\n"
-	"sweep speed ranging from 100 microseconds (usec) per division to 50\n"
-	"miliseconds (msec) per division.  There are 5 horizontal divisions,\n"
-	"so a trace sweep time can range from 500 usec (1/2000 sec) to\n"
-	"250 msec (1/4 sec).\n"),
+N_("<b>Panel Buttons\n"),
+N_("In oscilloscope mode, the two buttons to the left of the VU Meter select\n"
+	"an oscope horizontal sweep speed ranging from 100 microseconds (usec)\n"
+	"per division to 50 miliseconds (msec) per division.  There are 5\n"
+	"horizontal divisions, so a trace sweep time can range from 500 usec\n"
+	"(1/2000 sec) to 250 msec (1/4 sec).\n"),
+"\n",
+N_("In spectrum analyzer mode, the two buttons select a frequency resolution\n"
+	"and range.  As the '>' button is pressed, there is better lower\n"
+	"frequency resolution at the expense of slower update times.\n"),
 "\n",
 N_("<b>\nMouse Button Actions:\n"),
 N_("<b>\tLeft "),
 N_("click and drag in the VU Meter krell region to adjust VU Meter.\n"
 	"\t\tand chart sensitivity.\n"),
+N_("<b>\tLeft "),
+N_("on the chart in spectrum analyzer mode toggles a frequency\n"
+	"\t\tto draw highlighted.\n"),
+
 N_("<b>\tWheel "),
-N_("anywhere in panel also adjusts sensitivity.\n"),
+N_("button anywhere in the panel also adjusts sensitivity.\n"),
+N_("<b>\tMiddle "),
+N_("on the chart toggles display of chart labels.\n"),
 N_("<b>\tRight "),
-N_("on panel opens this configuration window.\n"),
-N_("\t\tOn chart toggles display of chart labels.\n")
+N_("on the panel opens this configuration window.\n")
 
 };
 
@@ -876,7 +708,7 @@ create_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellMSS %d.%d\nGKrellM Sound Scope\n\n"
-				"Copyright (c) 2001 by Bill Wilson\n"
+				"Copyright (c) 2002 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU Public License"),
@@ -890,12 +722,12 @@ create_tab(GtkWidget *tab_vbox)
 	}
 
 
-static Monitor	monitor_scope =
+static Monitor	monitor_sound =
 	{
 	N_(CONFIG_NAME),	/* Name, for config tab.	*/
 	0,					/* Id,  0 if a plugin		*/
-	create_scope,		/* The create function		*/
-	update_scope,		/* The update function		*/
+	create_sound,		/* The create function		*/
+	update_sound,		/* The update function		*/
 	create_tab,			/* The config tab create function	*/
 	apply_config,		/* Apply the config function		*/
 
@@ -916,18 +748,22 @@ static Monitor	monitor_scope =
 Monitor *
 init_plugin(void)
 	{
-	monitor_scope.name = _(monitor_scope.name);
-	mon_scope = &monitor_scope;
-
-	oscope = g_new0(Oscope, 1);
-	oscope->usec_per_div = 2000;
-	oscope->vert_sensitivity = 0.5;
-	oscope->vert_max = (gint) (oscope->vert_sensitivity * 32767.0);
+	monitor_sound.name = _(monitor_sound.name);
+	mon_sound = &monitor_sound;
 
 	sound = g_new0(SoundMonitor, 1);
 	sound->x_sensitivity_raw = gkrellm_chart_width() / 2;
+	sound->extra_info = TRUE;
+	sound->vert_sensitivity = 0.5;
 
-	style_id = gkrellm_add_meter_style(mon_scope, STYLE_NAME);
+	style_id = gkrellm_add_meter_style(mon_sound, STYLE_NAME);
 
-	return &monitor_scope;
+	init_oscope();
+	init_spectrum();
+
+	return &monitor_sound;
 	}
+
+/* check sensitivity yoff
+   left_margin == 0 => use vu_y0
+*/
