@@ -1,5 +1,5 @@
 /* GKrellM Sound Scope
-|  Copyright (C) 2001 Bill Wilson
+|  Copyright (C) 2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -19,23 +19,30 @@
 */
 
 #include <gkrellm/gkrellm.h>
-
+#include <math.h>
 #include <esd.h>
+#include <rfftw.h>
 
 #if  !defined(GKRELLM_VERSION_MAJOR) \
 	|| (GKRELLM_VERSION_MAJOR==1 && GKRELLM_VERSION_MINOR==0) \
 	|| (GKRELLM_VERSION_MAJOR==1 && GKRELLM_VERSION_MINOR==2 \
-		&& GKRELLM_VERSION_REV<2)
-#error This GKrellMSS plugin requires GKrellM version >= 1.2.2
+		&& GKRELLM_VERSION_REV<9)
+#error This GKrellMSS plugin requires GKrellM version >= 1.2.9
 #endif
 
 
 #define	GKRELLMSS_VERSION_MAJOR	0
-#define	GKRELLMSS_VERSION_MINOR	3
+#define	GKRELLMSS_VERSION_MINOR	4
 
+#define CONFIG_NAME     "Sound Scope"
+#define CONFIG_KEYWORD  "sound"
+#define STYLE_NAME      "sound"
 
-#define	SAMPLE_RATE	44100
-#define	N_SAMPLES	2048
+#define	DEBUG()			(GK.debug == 45)
+
+#define	SAMPLE_RATE		44100
+#define	N_SAMPLES		1024
+#define N_FFT_SAMPLES	8192
 
 #define	HORIZONTAL_DIVS	5
 #define	VERTICAL_DIVS	4
@@ -55,6 +62,7 @@ typedef enum
 	}
 	SoundStatus;
 
+
 typedef struct
 	{
 	short		left,
@@ -62,16 +70,23 @@ typedef struct
 	}
 SoundSample;
 
+
+#define	SOUND_MODE_OSCOPE	0
+#define	SOUND_MODE_SPECTRUM	1
+#define N_SOUND_MODES		2
+
+
 typedef struct
 	{
+	gint		mode;
+	GdkGC		*gc;
 	Krell		*krell_left_peak,
 				*krell_right_peak,
 				*krell_left,
 				*krell_right,
 				*krell_sensitivity;
 
-	gint
-				left_value,
+	gint		left_value,
 				right_value,
 				left_peak_value,
 				right_peak_value;
@@ -83,10 +98,16 @@ typedef struct
 				krell_sensitivity_y_up,
 				krell_sensitivity_y_dn;
 
+	gfloat		vert_sensitivity;
+
 	gchar		*host;
 	gint		fd;
 	gint		input_id;
-	gboolean	streaming;
+	gboolean	streaming,
+				extra_info,
+				mouse_in_chart,
+				mode_button_pressed;
+	gint		x_mouse;
 	SoundStatus	status;
 
 	gint		buf_count;
@@ -94,9 +115,42 @@ typedef struct
 	gint		buf_index;
 	SoundSample *buffer;
 	}
-SoundMonitor;
+	SoundMonitor;
 
-extern SoundMonitor	*sound;
+
+typedef struct
+	{
+	gint	start_bar,
+			x0_chart,
+			x0_bar,
+			dx_bar;
+
+	gint	*freq_array;
+	gint	array_size;
+	double	freq_quantum;
+
+	gint	n_samples;
+	fftw_plan *plan;
+	}
+	SpectrumScale;
+
+
+typedef struct
+	{
+	gint		fftw_samples;
+	fftw_real	*fftw_data_in,
+				*fftw_data_out,
+				*fftw_power_spectrum;
+	GdkPixmap	*bar,
+				*bar_light;
+	gint		scale_index;
+	SpectrumScale *scale;
+	gint		vert_max;
+	gint		freq_highlighted,
+				x_highlight;
+	gboolean	reset;
+	}
+	Spectrum;
 
 
 typedef struct
@@ -104,22 +158,45 @@ typedef struct
 	gint		usec_per_div,
 				vert_max,
 				dx_per_point;
-	gboolean	dirty;
-	gfloat		vert_sensitivity;
+	gboolean	dirty,
+				reset;
 	gfloat		vert_trigger;
 	gfloat		t_sample,
 				t_trace,
 				samples_per_point;
 	gint		x_append,
 				y_append;
-	GdkGC		*gc;
 	}
 Oscope;
 
-extern Chart	*chart;
-extern Oscope	*oscope;
+
+#define LOW_SWEEP   100
+#define HIGH_SWEEP  50000
+
+#define	FORCE_RESET	1
+#define	DRAW_GRID	1
+
+extern SoundMonitor	*sound;
+extern Chart		*chart;
+extern Oscope		*oscope;
+extern Spectrum		*spectrum;
+
+extern Decal		*decal_label0,
+					*decal_label1;
 
 extern SoundStatus	sound_status(gchar *);
 extern void			sound_open_stream(gchar *);
 extern void			sound_close_stream(void);
 extern void			oscope_trace(gint);
+
+/* oscope */
+void	draw_oscope(gboolean, gboolean);
+void	oscope_horizontal_scaling(void);
+void	init_oscope(void);
+
+
+/* Spectrum analyzer */
+void	draw_spectrum(gboolean, gboolean);
+void	load_spectrum_images(void);
+void	change_spectrum_scale(gint);
+void	init_spectrum(void);
